
/***************************************************************************
 *  DynamicReconfigureInterface.h - Fawkes BlackBoard Interface - DynamicReconfigureInterface
 *
 *  Templated created:   Thu Oct 12 10:49:19 2006
 *  Copyright  2017  Christoph Henke
 *
 ****************************************************************************/

/*  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version. A runtime exception applies to
 *  this software (see LICENSE.GPL_WRE file mentioned below for details).
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  Read the full text in the LICENSE.GPL_WRE file in the doc directory.
 */

#ifndef _INTERFACES_DYNAMICRECONFIGUREINTERFACE_H_
#define _INTERFACES_DYNAMICRECONFIGUREINTERFACE_H_

#include <interface/interface.h>
#include <interface/message.h>
#include <interface/field_iterator.h>

namespace fawkes {

class DynamicReconfigureInterface : public Interface
{
 /// @cond INTERNALS
 INTERFACE_MGMT_FRIENDS(DynamicReconfigureInterface)
 /// @endcond
 public:
  /* constants */

  /** Status of the last send message */
  typedef enum {
    Succeeded /**< Message was successful send */,
    Failed /**< Message failed */
  } LastMsgStatus;
  const char * tostring_LastMsgStatus(LastMsgStatus value) const;

 private:
  /** Internal data storage, do NOT modify! */
  typedef struct {
    int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
    int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    char last_service[64]; /**< The last service for reconfiguration called. */
    char last_parameter[64]; /**< The last parameter name. */
    bool last_bool_value; /**< The last parameter value. */
    char last_str_value[64]; /**< The last parameter value. */
    uint32_t last_uint32_value; /**< The last parameter value. */
    uint64_t last_uint64_value; /**< The last parameter value. */
    float last_float_value; /**< The last parameter value. */
    uint64_t last_msg_id; /**< The last parameter name. */
    int32_t last_msg_status; /**< The last send message status. */
  } DynamicReconfigureInterface_data_t;

  DynamicReconfigureInterface_data_t *data;

  interface_enum_map_t enum_map_LastMsgStatus;
 public:
  /* messages */
  class SetBoolMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char service[64]; /**< Name of the ROS service for dynamic reconfiguration. */
      char parameter[64]; /**< Name of the ROS parameter. */
      bool value; /**< The bool value. */
    } SetBoolMessage_data_t;

    SetBoolMessage_data_t *data;

  interface_enum_map_t enum_map_LastMsgStatus;
   public:
    SetBoolMessage(const char * ini_service, const char * ini_parameter, const bool ini_value);
    SetBoolMessage();
    ~SetBoolMessage();

    explicit SetBoolMessage(const SetBoolMessage *m);
    /* Methods */
    char * service() const;
    void set_service(const char * new_service);
    size_t maxlenof_service() const;
    char * parameter() const;
    void set_parameter(const char * new_parameter);
    size_t maxlenof_parameter() const;
    bool is_value() const;
    void set_value(const bool new_value);
    size_t maxlenof_value() const;
    virtual Message * clone() const;
  };

  class SetStringMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char service[64]; /**< Name of the ROS service for dynamic reconfiguration. */
      char parameter[64]; /**< Name of the ROS parameter. */
      char value[64]; /**< The value to set. */
    } SetStringMessage_data_t;

    SetStringMessage_data_t *data;

  interface_enum_map_t enum_map_LastMsgStatus;
   public:
    SetStringMessage(const char * ini_service, const char * ini_parameter, const char * ini_value);
    SetStringMessage();
    ~SetStringMessage();

    explicit SetStringMessage(const SetStringMessage *m);
    /* Methods */
    char * service() const;
    void set_service(const char * new_service);
    size_t maxlenof_service() const;
    char * parameter() const;
    void set_parameter(const char * new_parameter);
    size_t maxlenof_parameter() const;
    char * value() const;
    void set_value(const char * new_value);
    size_t maxlenof_value() const;
    virtual Message * clone() const;
  };

  class SetUint32Message : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char service[64]; /**< Name of the ROS service for dynamic reconfiguration. */
      char parameter[64]; /**< Name of the ROS parameter. */
      uint32_t value; /**< The value to set. */
    } SetUint32Message_data_t;

    SetUint32Message_data_t *data;

  interface_enum_map_t enum_map_LastMsgStatus;
   public:
    SetUint32Message(const char * ini_service, const char * ini_parameter, const uint32_t ini_value);
    SetUint32Message();
    ~SetUint32Message();

    explicit SetUint32Message(const SetUint32Message *m);
    /* Methods */
    char * service() const;
    void set_service(const char * new_service);
    size_t maxlenof_service() const;
    char * parameter() const;
    void set_parameter(const char * new_parameter);
    size_t maxlenof_parameter() const;
    uint32_t value() const;
    void set_value(const uint32_t new_value);
    size_t maxlenof_value() const;
    virtual Message * clone() const;
  };

  class SetUint64Message : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char service[64]; /**< Name of the ROS service for dynamic reconfiguration. */
      char parameter[64]; /**< Name of the ROS parameter. */
      uint64_t value; /**< The value to set. */
    } SetUint64Message_data_t;

    SetUint64Message_data_t *data;

  interface_enum_map_t enum_map_LastMsgStatus;
   public:
    SetUint64Message(const char * ini_service, const char * ini_parameter, const uint64_t ini_value);
    SetUint64Message();
    ~SetUint64Message();

    explicit SetUint64Message(const SetUint64Message *m);
    /* Methods */
    char * service() const;
    void set_service(const char * new_service);
    size_t maxlenof_service() const;
    char * parameter() const;
    void set_parameter(const char * new_parameter);
    size_t maxlenof_parameter() const;
    uint64_t value() const;
    void set_value(const uint64_t new_value);
    size_t maxlenof_value() const;
    virtual Message * clone() const;
  };

  class SetFloatMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char service[64]; /**< Name of the ROS service for dynamic reconfiguration. */
      char parameter[64]; /**< Name of the ROS parameter. */
      float value; /**< The value to set. */
    } SetFloatMessage_data_t;

    SetFloatMessage_data_t *data;

  interface_enum_map_t enum_map_LastMsgStatus;
   public:
    SetFloatMessage(const char * ini_service, const char * ini_parameter, const float ini_value);
    SetFloatMessage();
    ~SetFloatMessage();

    explicit SetFloatMessage(const SetFloatMessage *m);
    /* Methods */
    char * service() const;
    void set_service(const char * new_service);
    size_t maxlenof_service() const;
    char * parameter() const;
    void set_parameter(const char * new_parameter);
    size_t maxlenof_parameter() const;
    float value() const;
    void set_value(const float new_value);
    size_t maxlenof_value() const;
    virtual Message * clone() const;
  };

  virtual bool message_valid(const Message *message) const;
 private:
  DynamicReconfigureInterface();
  ~DynamicReconfigureInterface();

 public:
  /* Methods */
  char * last_service() const;
  void set_last_service(const char * new_last_service);
  size_t maxlenof_last_service() const;
  char * last_parameter() const;
  void set_last_parameter(const char * new_last_parameter);
  size_t maxlenof_last_parameter() const;
  bool is_last_bool_value() const;
  void set_last_bool_value(const bool new_last_bool_value);
  size_t maxlenof_last_bool_value() const;
  char * last_str_value() const;
  void set_last_str_value(const char * new_last_str_value);
  size_t maxlenof_last_str_value() const;
  uint32_t last_uint32_value() const;
  void set_last_uint32_value(const uint32_t new_last_uint32_value);
  size_t maxlenof_last_uint32_value() const;
  uint64_t last_uint64_value() const;
  void set_last_uint64_value(const uint64_t new_last_uint64_value);
  size_t maxlenof_last_uint64_value() const;
  float last_float_value() const;
  void set_last_float_value(const float new_last_float_value);
  size_t maxlenof_last_float_value() const;
  uint64_t last_msg_id() const;
  void set_last_msg_id(const uint64_t new_last_msg_id);
  size_t maxlenof_last_msg_id() const;
  LastMsgStatus last_msg_status() const;
  void set_last_msg_status(const LastMsgStatus new_last_msg_status);
  size_t maxlenof_last_msg_status() const;
  virtual Message * create_message(const char *type) const;

  virtual void copy_values(const Interface *other);
  virtual const char * enum_tostring(const char *enumtype, int val) const;

};

} // end namespace fawkes

#endif
