
/***************************************************************************
 *  HardwareModelsInterface.h - Fawkes BlackBoard Interface - HardwareModelsInterface
 *
 *  Templated created:   Thu Oct 12 10:49:19 2006
 *  Copyright  2019  Daniel Habering
 *
 ****************************************************************************/

/*  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version. A runtime exception applies to
 *  this software (see LICENSE.GPL_WRE file mentioned below for details).
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  Read the full text in the LICENSE.GPL_WRE file in the doc directory.
 */

#ifndef _INTERFACES_HARDWAREMODELSINTERFACE_H_
#define _INTERFACES_HARDWAREMODELSINTERFACE_H_

#include <interface/interface.h>
#include <interface/message.h>
#include <interface/field_iterator.h>

namespace fawkes {

class HardwareModelsInterface : public Interface
{
 /// @cond INTERNALS
 INTERFACE_MGMT_FRIENDS(HardwareModelsInterface)
 /// @endcond
 public:
  /* constants */

 private:
  /** Internal data storage, do NOT modify! */
  typedef struct {
    int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
    int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    char error[1024]; /**< Error state of the plugin */
    bool busy; /**< Busy flag */
  } HardwareModelsInterface_data_t;

  HardwareModelsInterface_data_t *data;

 public:
  /* messages */
  class StateChangeMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char component[1024]; /**< The component that changed its state */
      char transition[1024]; /**< The transition executed by the component */
    } StateChangeMessage_data_t;

    StateChangeMessage_data_t *data;

   public:
    StateChangeMessage(const char * ini_component, const char * ini_transition);
    StateChangeMessage();
    ~StateChangeMessage();

    explicit StateChangeMessage(const StateChangeMessage *m);
    /* Methods */
    char * component() const;
    void set_component(const char * new_component);
    size_t maxlenof_component() const;
    char * transition() const;
    void set_transition(const char * new_transition);
    size_t maxlenof_transition() const;
    virtual Message * clone() const;
  };

  virtual bool message_valid(const Message *message) const;
 private:
  HardwareModelsInterface();
  ~HardwareModelsInterface();

 public:
  /* Methods */
  char * error() const;
  void set_error(const char * new_error);
  size_t maxlenof_error() const;
  bool is_busy() const;
  void set_busy(const bool new_busy);
  size_t maxlenof_busy() const;
  virtual Message * create_message(const char *type) const;

  virtual void copy_values(const Interface *other);
  virtual const char * enum_tostring(const char *enumtype, int val) const;

};

} // end namespace fawkes

#endif
