
/***************************************************************************
 *  LaserClusterInterface.h - Fawkes BlackBoard Interface - LaserClusterInterface
 *
 *  Templated created:   Thu Oct 12 10:49:19 2006
 *  Copyright  2013  Tim Niemueller
 *
 ****************************************************************************/

/*  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version. A runtime exception applies to
 *  this software (see LICENSE.GPL_WRE file mentioned below for details).
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  Read the full text in the LICENSE.GPL_WRE file in the doc directory.
 */

#ifndef _INTERFACES_LASERCLUSTERINTERFACE_H_
#define _INTERFACES_LASERCLUSTERINTERFACE_H_

#include <interface/interface.h>
#include <interface/message.h>
#include <interface/field_iterator.h>

namespace fawkes {

class LaserClusterInterface : public Interface
{
 /// @cond INTERNALS
 INTERFACE_MGMT_FRIENDS(LaserClusterInterface)
 /// @endcond
 public:
  /* constants */

  /** 
        Enumeration defining the possible cluster selection modes.
       */
  typedef enum {
    SELMODE_MIN_ANGLE /**< 
	Choose the cluster with the minimum angle difference from 0 degrees.
       */,
    SELMODE_MIN_DIST /**< 
	Choose the cluster with the minimum distance in X direction of
	the reference frame (typically forward).
       */
  } SelectionMode;
  const char * tostring_SelectionMode(SelectionMode value) const;

 private:
  /** Internal data storage, do NOT modify! */
  typedef struct {
    int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
    int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    float max_x; /**< Maximum distance in X coordinate
    of sensor frame. */
    int32_t selection_mode; /**< 
      Current cluster selection mode.
     */
  } LaserClusterInterface_data_t;

  LaserClusterInterface_data_t *data;

  interface_enum_map_t enum_map_SelectionMode;
 public:
  /* messages */
  class SetMaxXMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      float max_x; /**< Maximum distance in X coordinate
    of sensor frame. */
    } SetMaxXMessage_data_t;

    SetMaxXMessage_data_t *data;

  interface_enum_map_t enum_map_SelectionMode;
   public:
    SetMaxXMessage(const float ini_max_x);
    SetMaxXMessage();
    ~SetMaxXMessage();

    explicit SetMaxXMessage(const SetMaxXMessage *m);
    /* Methods */
    float max_x() const;
    void set_max_x(const float new_max_x);
    size_t maxlenof_max_x() const;
    virtual Message * clone() const;
  };

  class SetSelectionModeMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      int32_t selection_mode; /**< 
      Current cluster selection mode.
     */
    } SetSelectionModeMessage_data_t;

    SetSelectionModeMessage_data_t *data;

  interface_enum_map_t enum_map_SelectionMode;
   public:
    SetSelectionModeMessage(const SelectionMode ini_selection_mode);
    SetSelectionModeMessage();
    ~SetSelectionModeMessage();

    explicit SetSelectionModeMessage(const SetSelectionModeMessage *m);
    /* Methods */
    SelectionMode selection_mode() const;
    void set_selection_mode(const SelectionMode new_selection_mode);
    size_t maxlenof_selection_mode() const;
    virtual Message * clone() const;
  };

  virtual bool message_valid(const Message *message) const;
 private:
  LaserClusterInterface();
  ~LaserClusterInterface();

 public:
  /* Methods */
  float max_x() const;
  void set_max_x(const float new_max_x);
  size_t maxlenof_max_x() const;
  SelectionMode selection_mode() const;
  void set_selection_mode(const SelectionMode new_selection_mode);
  size_t maxlenof_selection_mode() const;
  virtual Message * create_message(const char *type) const;

  virtual void copy_values(const Interface *other);
  virtual const char * enum_tostring(const char *enumtype, int val) const;

};

} // end namespace fawkes

#endif
