
/***************************************************************************
 *  ProtobufPeerInterface.h - Fawkes BlackBoard Interface - ProtobufPeerInterface
 *
 *  Templated created:   Thu Oct 12 10:49:19 2006
 *  Copyright  2017  Victor Mataré
 *
 ****************************************************************************/

/*  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version. A runtime exception applies to
 *  this software (see LICENSE.GPL_WRE file mentioned below for details).
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  Read the full text in the LICENSE.GPL_WRE file in the doc directory.
 */

#ifndef _INTERFACES_PROTOBUFPEERINTERFACE_H_
#define _INTERFACES_PROTOBUFPEERINTERFACE_H_

#include <interface/interface.h>
#include <interface/message.h>
#include <interface/field_iterator.h>

namespace fawkes {

class ProtobufPeerInterface : public Interface
{
 /// @cond INTERNALS
 INTERFACE_MGMT_FRIENDS(ProtobufPeerInterface)
 /// @endcond
 public:
  /* constants */

 private:
  /** Internal data storage, do NOT modify! */
  typedef struct {
    int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
    int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
    int64_t peers[16]; /**< Currently active peers */
  } ProtobufPeerInterface_data_t;

  ProtobufPeerInterface_data_t *data;

 public:
  /* messages */
  class CreatePeerMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char address[255]; /**< IP address or host name */
      int32_t port; /**< Port to send to/receive on */
    } CreatePeerMessage_data_t;

    CreatePeerMessage_data_t *data;

   public:
    CreatePeerMessage(const char * ini_address, const int32_t ini_port);
    CreatePeerMessage();
    ~CreatePeerMessage();

    explicit CreatePeerMessage(const CreatePeerMessage *m);
    /* Methods */
    char * address() const;
    void set_address(const char * new_address);
    size_t maxlenof_address() const;
    int32_t port() const;
    void set_port(const int32_t new_port);
    size_t maxlenof_port() const;
    virtual Message * clone() const;
  };

  class CreatePeerLocalMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char address[255]; /**< IP address or host name */
      int32_t send_to_port; /**< Port to send to */
      int32_t recv_on_port; /**< Port to receive on */
    } CreatePeerLocalMessage_data_t;

    CreatePeerLocalMessage_data_t *data;

   public:
    CreatePeerLocalMessage(const char * ini_address, const int32_t ini_send_to_port, const int32_t ini_recv_on_port);
    CreatePeerLocalMessage();
    ~CreatePeerLocalMessage();

    explicit CreatePeerLocalMessage(const CreatePeerLocalMessage *m);
    /* Methods */
    char * address() const;
    void set_address(const char * new_address);
    size_t maxlenof_address() const;
    int32_t send_to_port() const;
    void set_send_to_port(const int32_t new_send_to_port);
    size_t maxlenof_send_to_port() const;
    int32_t recv_on_port() const;
    void set_recv_on_port(const int32_t new_recv_on_port);
    size_t maxlenof_recv_on_port() const;
    virtual Message * clone() const;
  };

  class CreatePeerCryptoMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char address[255]; /**< IP address or host name */
      int32_t port; /**< Port to send to/receive on */
      char crypto_key[1024]; /**< Crypto key */
      char cipher[255]; /**< Cipher name */
    } CreatePeerCryptoMessage_data_t;

    CreatePeerCryptoMessage_data_t *data;

   public:
    CreatePeerCryptoMessage(const char * ini_address, const int32_t ini_port, const char * ini_crypto_key, const char * ini_cipher);
    CreatePeerCryptoMessage();
    ~CreatePeerCryptoMessage();

    explicit CreatePeerCryptoMessage(const CreatePeerCryptoMessage *m);
    /* Methods */
    char * address() const;
    void set_address(const char * new_address);
    size_t maxlenof_address() const;
    int32_t port() const;
    void set_port(const int32_t new_port);
    size_t maxlenof_port() const;
    char * crypto_key() const;
    void set_crypto_key(const char * new_crypto_key);
    size_t maxlenof_crypto_key() const;
    char * cipher() const;
    void set_cipher(const char * new_cipher);
    size_t maxlenof_cipher() const;
    virtual Message * clone() const;
  };

  class CreatePeerLocalCryptoMessage : public Message
  {
   private:
    /** Internal data storage, do NOT modify! */
    typedef struct {
      int64_t timestamp_sec;  /**< Interface Unix timestamp, seconds */
      int64_t timestamp_usec; /**< Interface Unix timestamp, micro-seconds */
      char address[255]; /**< IP address or host name */
      int32_t send_to_port; /**< Port to send to */
      int32_t recv_on_port; /**< Port to receive on */
      char crypto_key[1024]; /**< Crypto key */
      char cipher[255]; /**< Cipher name */
    } CreatePeerLocalCryptoMessage_data_t;

    CreatePeerLocalCryptoMessage_data_t *data;

   public:
    CreatePeerLocalCryptoMessage(const char * ini_address, const int32_t ini_send_to_port, const int32_t ini_recv_on_port, const char * ini_crypto_key, const char * ini_cipher);
    CreatePeerLocalCryptoMessage();
    ~CreatePeerLocalCryptoMessage();

    explicit CreatePeerLocalCryptoMessage(const CreatePeerLocalCryptoMessage *m);
    /* Methods */
    char * address() const;
    void set_address(const char * new_address);
    size_t maxlenof_address() const;
    int32_t send_to_port() const;
    void set_send_to_port(const int32_t new_send_to_port);
    size_t maxlenof_send_to_port() const;
    int32_t recv_on_port() const;
    void set_recv_on_port(const int32_t new_recv_on_port);
    size_t maxlenof_recv_on_port() const;
    char * crypto_key() const;
    void set_crypto_key(const char * new_crypto_key);
    size_t maxlenof_crypto_key() const;
    char * cipher() const;
    void set_cipher(const char * new_cipher);
    size_t maxlenof_cipher() const;
    virtual Message * clone() const;
  };

  virtual bool message_valid(const Message *message) const;
 private:
  ProtobufPeerInterface();
  ~ProtobufPeerInterface();

 public:
  /* Methods */
  int64_t * peers() const;
  int64_t peers(unsigned int index) const;
  void set_peers(unsigned int index, const int64_t new_peers);
  void set_peers(const int64_t * new_peers);
  size_t maxlenof_peers() const;
  virtual Message * create_message(const char *type) const;

  virtual void copy_values(const Interface *other);
  virtual const char * enum_tostring(const char *enumtype, int val) const;

};

} // end namespace fawkes

#endif
