
/***************************************************************************
 *  url_manager.h - Web URL manager
 *
 *  Created: Thu Nov 25 21:53:07 2010
 *  Copyright  2006-2010  Tim Niemueller [www.niemueller.de]
 *
 ****************************************************************************/

/*  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  Read the full text in the LICENSE.GPL file in the doc directory.
 */

#ifndef _LIBS_WEBVIEW_URL_MANAGER_H_
#define _LIBS_WEBVIEW_URL_MANAGER_H_

#include <webview/request.h>
#include <webview/router.h>

#include <list>
#include <mutex>

namespace fawkes {

class Mutex;
class WebReply;
template <typename T>
class WebviewRouter;

class WebUrlManager
{
	friend WebRequestDispatcher;

public:
	/** Function type for handling requests. */
	typedef std::function<WebReply *(const WebRequest *)> Handler;

	WebUrlManager();
	~WebUrlManager();

	void add_handler(WebRequest::Method method, const std::string &path, Handler handler);
	void remove_handler(WebRequest::Method method, const std::string &path);

	void add_handler(WebRequest::Method method, const std::string &path, Handler handler, int weight);

private:
	WebReply *process_request(WebRequest *request);

private:
	std::mutex                              mutex_;
	std::shared_ptr<WebviewRouter<Handler>> router_;
};

} // end namespace fawkes

#endif
