#*****************************************************************************
#   Makefile Build System for Fawkes: Config Settings specific to Fawkes
#                            -------------------
#   Created on Thu Oct 16 20:00:11 2008
#   Copyright (C) 2006-2010 by Tim Niemueller, AllemaniACs RoboCup Team
#
#*****************************************************************************
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#*****************************************************************************

include $(BUILDSYSDIR)/btypes/config_fawkes.mk

# Not silent, no color
SILENT  = @
COLORED = 0

# Can be enabled via CFLAGS
GCC_USE_OPENMP=0
DO_32BIT_BUILD=0

# For 64-bit architectures we must append 64 to lib dir
ifneq ($(filter $(ARCH),x86_64 aarch64 ppc64 ppc64le s390x),)
  LIBBITS=64
endif

DESTDIR        =
PREFIX         = /usr/local
SYSCONFDIR     = /etc/fawkes

# Base CFLAGS, LDFLAGS
# - use RPATH for interfaces libdir, required for proper linking while still
#   separating the (many) interface libs from the normal libdir and avoiding
#   the need to add an entry per command to /etc/ld.so.conf.d/ (or similar)
CFLAGS_BASE    = $(CFLAGS_MINIMUM)
LDFLAGS_BASE   = $(LDFLAGS_MINIMUM) -Wl,-rpath=$(EXEC_LIBDIR)/fawkes/interfaces

# Note: the install scripts make the assumption that FFLIBDIR, SHAREDIR and INCDIR
# paths (with EXEC_ prefix) are specific to Fawkes and can be deleted completely
# without interfering with the rest of the system!
FFLIBSUBDIR        = fawkes
EXEC_BASEDIR       = $(abspath $(PREFIX))
EXEC_CONFDIR       = $(SYSCONFDIR)
EXEC_SHAREDIR      = $(EXEC_BASEDIR)/share/fawkes
EXEC_RESDIR        = $(EXEC_SHAREDIR)/res
EXEC_LIBDIR        = $(EXEC_BASEDIR)/lib$(LIBBITS)
EXEC_FFLIBDIR      = $(EXEC_LIBDIR)/$(FFLIBSUBDIR)
EXEC_PLUGINDIR     = $(EXEC_FFLIBDIR)/plugins
EXEC_IFACEDIR      = $(EXEC_FFLIBDIR)/interfaces
EXEC_INCDIR        = $(EXEC_BASEDIR)/include/fawkes
EXEC_DFILEDIR      = $(EXEC_BASEDIR)/share/applications
EXEC_DOCDIR        = $(EXEC_BASEDIR)/share/doc/fawkes-$(FAWKES_VERSION)
EXEC_MANDIR        = $(EXEC_BASEDIR)/share/man
EXEC_LUADIR        = $(EXEC_SHAREDIR)/lua
EXEC_LUALIBDIR     = $(EXEC_FFLIBDIR)/lua
EXEC_BUILDSYSDIR   = $(EXEC_SHAREDIR)/buildsys
EXEC_BUILDCONFDIR  = $(EXEC_BUILDSYSDIR)/conf

FILE_INSTALL = install

# The default file mode used for additional install files
# (using INSTALL_extra targets)
FILEMODE_DEFAULT   = 0644

ifneq ($(CFLAGS_EXT),)
  CFLAGS_BASE += $(CFLAGS_EXT)
endif
ifneq ($(LDFLAGS_EXT),)
  LDFLAGS_BASE += $(LDFLAGS_EXT)
endif
